﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.IO.IsolatedStorage;
using System.ComponentModel;
using Microsoft.Phone.Controls;
using Microsoft.Phone.Tasks;
using System.Device.Location;
using System.Threading;

namespace GPS_Distance
{
    public class GeneralCommand : ICommand
    {
        private Action<object> execute;
        private bool currentCanExecute = false;
        public event EventHandler CanExecuteChanged;
        public bool CanExecute(object parameter)
        {
            return currentCanExecute;
        }
        public void NotifyCanExecuteChanged(bool newState)
        {
            bool oldState = currentCanExecute;
            currentCanExecute = newState;
            if (newState != oldState) CanExecuteChanged(this, new EventArgs());
        }
        public void Execute(object parameter)
        {
            execute(parameter);
        }
        public GeneralCommand(Action<object> execute, bool initialState = true)
        {
            this.execute = execute;
            this.currentCanExecute = initialState;
        }
    }

    public class MainViewModel : INotifyPropertyChanged
    {
        private bool isDataLoaded;
        public bool IsDataLoaded
        {
            get { return isDataLoaded; }
            set
            {
                isDataLoaded = value;
                NotifyPropertyChanged("IsDataLoaded");
            }
        }
        private string gpsStatus;
        public string GpsStatus
        {
            get { return gpsStatus; }
            set
            {
                gpsStatus = value;
                NotifyPropertyChanged("GpsStatus");
            }
        }
        private string gpsCount;
        public string GpsCount
        {
            get { return gpsCount; }
            set
            {
                gpsCount = value;
                NotifyPropertyChanged("GpsCount");
            }
        }
        private string distance = "???";
        public string Distance
        {
            get { return distance; }
            set
            {
                distance = value;
                NotifyPropertyChanged("Distance");
            }
        }
        public event PropertyChangedEventHandler PropertyChanged;
        private void NotifyPropertyChanged(String propertyName)
        {
            PropertyChangedEventHandler handler = PropertyChanged;
            if (null != handler)
            {
                handler(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        private double tryGetDouble(string key, double defValue)
        {
            if (IsolatedStorageSettings.ApplicationSettings.Contains(key))
                return (double)IsolatedStorageSettings.ApplicationSettings[key];
            else
                return defValue;
        }

        private GeoCoordinate basepoint
        {
            get
            {
                return new GeoCoordinate(tryGetDouble("lat", 35.6811), tryGetDouble("long", 139.7668));
            }
            set
            {
                IsolatedStorageSettings.ApplicationSettings["lat"] = value.Latitude;
                IsolatedStorageSettings.ApplicationSettings["long"] = value.Longitude;
            }
        }

        private double distanceDouble
        {
            get
            {
                bool f = false;
                if (IsolatedStorageSettings.ApplicationSettings.Contains(Settings.DistMethodKey))
                    f = (bool)IsolatedStorageSettings.ApplicationSettings[Settings.DistMethodKey];
                if (f) return Yahoo.Distance(current,basepoint);
                else return current.GetDistanceTo(basepoint);
            }
        }

        private GeoCoordinateWatcher watcher;
        private GeoCoordinate current = new GeoCoordinate();
        private int counter;
#if false
        private Thread thread;
#endif
        public void InitWatcher()
        {
            watcher = new GeoCoordinateWatcher(GeoPositionAccuracy.High);
            watcher.PositionChanged += new EventHandler<GeoPositionChangedEventArgs<GeoCoordinate>>((sender2, e2) =>
            {
                current = e2.Position.Location;
                Distance = string.Format("{0:0.0}", distanceDouble / 1000);
                counter++;
                GpsCount = "◎○"[counter % 2].ToString();
            });
            watcher.StatusChanged += new EventHandler<GeoPositionStatusChangedEventArgs>((sender2, e2) =>
            {
                GpsStatus = e2.Status.ToString();
                switch (e2.Status)
                {
                    case GeoPositionStatus.Disabled:
                        break;
                    case GeoPositionStatus.Initializing:
#if false
                        thread = new Thread(() =>
                            {
                                Thread.Sleep(10000);
                                if (GpsStatus == "Initializing")
                                {
                                    MainPage.MyDispatcher.BeginInvoke(() =>
                                        {
                                            this.watcher.Dispose();
                                            InitWatcher();
                                        });
                                }
                            });
                        thread.Start();
#endif
                        break;
                    case GeoPositionStatus.NoData:
                        break;
                    case GeoPositionStatus.Ready:
                        break;
                }
            });
            watcher.Start();
        }

        public GeneralCommand ShowCurrentMapCommand { get; set; }
        public GeneralCommand ShowBaseMapCommand { get; set; }
        public GeneralCommand SetBasePointCommand { get; set; }

        private void showMap(GeoCoordinate coordinate)
        {
            BingMapsTask bingMapsTask = new BingMapsTask();
            bingMapsTask.Center = coordinate;
            bingMapsTask.ZoomLevel = 18;
            bingMapsTask.Show();
        }

        internal void LoadData()
        {
            InitWatcher();
        }
        public MainViewModel()
        {
            ShowCurrentMapCommand = new GeneralCommand(obj =>
            {
                if (current.IsUnknown)
                {
                    MessageBox.Show("座標が未知です。地図で表示できません。");
                    return;
                }
                showMap(current);
            });
            ShowBaseMapCommand = new GeneralCommand(obj =>
            {
                showMap(basepoint);
            });
            SetBasePointCommand = new GeneralCommand(obj =>
            {
                if (current.IsUnknown)
                {
                    MessageBox.Show("座標が未知です。設定できません。");
                    return;
                }
                if (MessageBox.Show("座標を現在位置に設定してよろしいですか?", "GPS距離計", MessageBoxButton.OKCancel) == MessageBoxResult.OK)
                {
                    basepoint = current;
                }
            });
        }
    }
}
