﻿/// <reference path="http://js.live.net/v5.0/wl.js" />
var keyHeader = "D";
var optionHeader = "LocalFileList";
var dataheader = "data:image/jpeg;base64,";
var hexletters = "0123456789abcdef";
var safeStringPrefix = "Plt2";  // 長さは４文字
var currentPropertyObject;
var overriteImage;
var updateImage;
var pixelsize = 200;

var yahoomap;
var currentFileId = null;
var currentPlots = null;

function parseArg() {
    var arg = new Object;
    var pair = location.search.substring(1).split('&');
    for (i = 0; pair[i]; i++) {
        var kv = pair[i].split('=');
        arg[kv[0]] = kv[1];
    }
    return arg;
}

function parseAuth() {
    var arg = new Object;
    if (!$.cookie('wl_auth')) return arg;
    var pair = $.cookie('wl_auth').split('&');
    for (i = 0; pair[i]; i++) {
        var kv = pair[i].split('=');
        arg[kv[0]] = kv[1];
    }
    return arg;
}

function loadingIndicatorOn() {
    $("#loading").css("visibility", "visible");
    $("#disableMask").css("visibility", "visible");
}

function loadingIndicatorOff() {
    $("#loading").css("visibility", "hidden");
    $("#disableMask").css("visibility", "hidden");
}

function hex1(n) {
    return hexletters.charAt(n % 16);
}

function hex4(n) {
    return hex1(n / 4096) + hex1(n / 256) + hex1(n / 16) + hex1(n);
}

function encodeSafeString(s) {
    var r = "";
    for (var i = 0; i < s.length; i++) {
        r += hex4(s.charCodeAt(i));
    }
    return safeStringPrefix+r;
}

function decodeSafeString(s) {
    var r = "";
    s = s.substring(4, s.length);
    for (var i = 0; i < s.length; i += 4) {
        var n = 0;
        for (var j = 0; j < 4; j++) {
            n *= 16;
            n += hexletters.indexOf(s.substring(i + j, i + j + 1));
        }
        r += String.fromCharCode(n);
    }
    return r;
}

function isSafeString(s) {
    // 4の倍数
    if (s.length % 4 != 0) return false;
    if (s.substring(0, 4) != safeStringPrefix) return false;
    s = s.substring(4, s.length);
    // 全て16進文字
    for (var i = 0; i < s.length; i++) {
        if (hexletters.indexOf(s.substring(i, i + 1)) < 0) return false;
    }
    return true;
}

function createDefaultAlterName() {
    function fn(v, d) {
        var s = "000" + v;
        return s.substr(s.length - d, s.length);
    }
    var now = new Date();
    return fn(now.getFullYear(), 4) + fn(now.getMonth() + 1, 2) + fn(now.getDate(), 2) + fn(now.getHours(), 2) + fn(now.getMinutes(), 2) + fn(now.getSeconds(), 2);
}

function nameFixer(s) {
    if (s && s != "") return s;
    return createDefaultAlterName();
}

function propertyOn(pntObj) {
    currentPropertyObject = pntObj;
    $("#propertyTitle").text(pntObj.name);
    $("#propertyDescription").html(pntObj.description);
    if (pntObj.Jpeg && pntObj.Jpeg.Text) {
        $("#propertyImage").attr("src", "data:image/jepg;base64," + pntObj.Jpeg.Text);
        $("#propertyImage").show();
    }
    else
        $("#propertyImage").hide();
    $("#propertyMask").css("visibility", "visible");
}

function propertyOff() {
    $("#propertyImage").hide();
    $("#propertyMask").css("visibility", "hidden");
}

function editOn(pntObj) {
    currentPropertyObject = pntObj;
    overriteImage = null;
    updateImage = false;
    $("#editTitle").val(pntObj.name);
    $("#editDescription").empty();
    $("#editDescription").append(pntObj.description);
    if (pntObj.Jpeg && pntObj.Jpeg.Text) {
        $("#editImage").attr("src", dataheader + pntObj.Jpeg.Text);
        $("#editImage").show();
    }
    else
        $("#editImage").hide();
    $("#editMask").css("visibility", "visible");
}

function editSave() {
    pntObj = currentPropertyObject;
    pntObj.name = $("#editTitle").val();
    pntObj.description = $("#editDescription").val();
    if (updateImage) {
        if (!pntObj.Jpeg) pntObj.Jpeg = {};
        pntObj.Jpeg.Text = overriteImage;
    }
}

function adjustImageSize(img,ctx) {
    var w = pixelsize;
    var h = pixelsize;
    if (img.width > img.height) {
        w = pixelsize;
        h = img.height * pixelsize / img.width;
        y = (pixelsize - h) / 2;
        ctx.drawImage(img, 0, y, w, h);
    }
    else {
        w = img.width * pixelsize / img.height;
        h = pixelsize;
        x = (pixelsize - w) / 2;
        ctx.drawImage(img, x, 0, w, h);
    }
}

function editUpload() {
    if (!window.FileReader) {
        alert("This browser is not support FileReader. Please try New Firefox, Chrome or other HTML5 Browsers.");
        return;
    }
    var file = document.getElementById("myFile").files[0];
    if (!file) {
        alert("Set file-name first");
        return;
    }
    var url_reader = new FileReader();
    url_reader.onload = function (evt) {
        if (evt.target.result.substring(0, dataheader.length) != dataheader) {
            alert("Please select JPEG file.");
            return;
        }
        var can = document.getElementById("workercanvas");
        var ctx = can.getContext('2d');
        var img = new Image();
        ctx.beginPath();
        ctx.fillRect(0, 0, pixelsize, pixelsize);
        img.onload = function () {
            adjustImageSize(img, ctx);
            var data = can.toDataURL("image/jpeg");
            if (data.substring(0, dataheader.length) != dataheader) {
                alert("This browser doesn't support CANVAS to JPEG Conversion.");
                return;
            }
            updateImage = true;
            overriteImage = data.substring(dataheader.length, data.length);
            $("#editImage").attr("src", data);
            $("#editImage").show();
        }
        img.src = evt.target.result;
    };
    url_reader.readAsDataURL(file);
}

function editDelete() {
    updateImage = true;
    overriteImage = null;
    $("#editImage").hide();
}

function editOff() {
    $("#editImage").hide();
    $("#editMask").css("visibility", "hidden");
}

function moveOn() {
    $("#moveWarn").css("visibility", "visible");
}

function moveOff() {
    $("#moveWarn").css("visibility", "hidden");
}

function isMoveOn() {
    return $("#moveWarn").css("visibility") == "visible";
}

function jpegOn() {
    $("#jpegMask").css("visibility", "visible");
}

function jpegOff() {
    $("#jpegMask").css("visibility", "hidden");
}

var jpegLoadQueue = [];

function jpegLocationLoaded(image, data) {
    try
    {
        var json = eval("(" + data + ")");
        if (json.lat && json.long) {
            var ll = new Y.LatLng(json.lat, json.long);
            var pntObj = addPoint(ll);
            if (!pntObj.Jpeg) pntObj.Jpeg = {};
            pntObj.Jpeg.Text = image;
        }
        else {
            alert("Can't get a location from Jpeg file.");
        }
    }
    catch(x)
    {
        alert("Can't get a location from Jpeg file.");
    }
    if (jpegLoadQueue.length <= 0) {
        jpegOff();
        loadingIndicatorOff();
        $("#jpegStatus").hide();
        yahoomap.drawMap(ll, 17, Y.LayerSetId.NORMAL);
    }
    else {
        jpegUploadForEachFile();
    }
}

function jpegUploadForEachFile() {
    $("#jpegStatus").text(jpegLoadQueue.length);
    $("#jpegStatus").show();
    var file = jpegLoadQueue.pop();
    var url_reader = new FileReader();
    url_reader.onload = function (evt) {
        if (evt.target.result.substring(0, dataheader.length) != dataheader) {
            alert("Please select JPEG file.");
            return;
        }
        var can = document.getElementById("workercanvas");
        var ctx = can.getContext('2d');
        var img = new Image();
        ctx.beginPath();
        ctx.fillRect(0, 0, pixelsize, pixelsize);
        img.onload = function () {
            adjustImageSize(img, ctx);
            var data = can.toDataURL("image/jpeg");
            if (data.substring(0, dataheader.length) != dataheader) {
                alert("This browser doesn't support CANVAS to JPEG Conversion.");
                return;
            }
            var image = data.substring(dataheader.length, data.length);
            $.post('jpeg.aspx', "jpeg=" + encodeURIComponent(evt.target.result.substring(dataheader.length, evt.target.result.length)), function (data) {
                jpegLocationLoaded(image, data);
            })
            .error(function (status) {
                if (status.status == 200) {
                    jpegLocationLoaded(image, status.responseText);
                } else {
                    alert("error " + status.status + "," + status.statusText);
                }
            });
        }
        img.src = evt.target.result;
    };
    url_reader.readAsDataURL(file);
}

function jpegUpload() {
    if (!window.FileReader) {
        alert("This browser is not support FileReader. Please try New Firefox, Chrome or other HTML5 Browsers.");
        return;
    }
    var jpegFile = document.getElementById("jpegFile");
    if (jpegFile.files.length <= 0) {
        alert("Set file-name first");
        return;
    }
    for (var i = 0; i < jpegFile.files.length; i++) jpegLoadQueue.push(jpegFile.files[i]);
    jpegLoadQueue.reverse();
    loadingIndicatorOn();
    jpegUploadForEachFile();
}

function onclickMarker(pntObj) {
    propertyOn(pntObj);
}

function addMarker2(pntObj) {
    var v = pntObj.Point.coordinates.split(",");
    var m = new Y.Marker(new Y.LatLng(v[1], v[0]), { "title": pntObj.name });
    yahoomap.addFeature(m);
    $(m.getDOMNode()).click(function () {
        onclickMarker(pntObj);
    });
}

function centerrize(lat,lng) {
    yahoomap.drawMap(new Y.LatLng(lng, lat), 17, Y.LayerSetId.NORMAL);
}

function getAccessToken() {
    return parseAuth()["access_token"];
}

function initForSkyDrive() {
    var id = "(replace your sky-drive developper id)";
    WL.init({ "client_id": id, "display": "touch" });
}
function loginForRead(func) {
    initForSkyDrive();
    WL.login(
     {
         "scope": "wl.skydrive"
     },
     function (response) {
         if (func) func(response);
     });
}

function loginForWrite(func) {
    initForSkyDrive();
    WL.login(
     {
         "scope": "wl.skydrive_update"
     },
     function (response) {
         if (func) func(response);
     });
}

function saveClose() {
    $("#savemask").css("visibility", "hidden");
}

function loadClose() {
    $("#loadmask").css("visibility", "hidden");
}

function commandsClose() {
    $("#toolPalette").css("visibility", "Hidden");
    $("#toolPalette2").css("visibility", "Hidden");
}

function createPlotierFolder(completion) {
    WL.api({
        path: "me/skydrive",
        method: "POST",
        body: {
            "name": "Plotier002",
            "description": "Folder for Plotier002 Files"
        }
    }).then(
                function (response) {
                    completion(response.id);
                },
                function (responseFailed) {
                    alert("Error calling API: " + responseFailed.error.message);
                    completion(null);
                }
            );
}

function getFolderId(completion) {
    loginForWrite(function (response) {
        if (response.status == "connected") {
            WL.api("me/skydrive/files", "GET", function (r2) {
                for (var i = 0; i < r2.data.length; i++) {
                    if (r2.data[i].name == "Plotier002") 
                        return completion(r2.data[i].id);
                }
                return createPlotierFolder(completion);
            });
        }
        else {
            alert("Could not connect, status = " + response.status);
        }
    });
    return null;
}

function initSkyDriveFileList(folderid) {
    $("#LocalFileList").children().remove();

    WL.api({
        path: folderid + "/files",
        method: "GET"
    }).then(function (result) {
        for (var i = 0; i < result.data.length; i++) {
            var n = result.data[i].name;
            if (n.substring(n.length - 4, n.length).toLowerCase() != ".txt") continue;
            var id = result.data[i].id;
            var namebody = n.substring(0, n.length - 4);    // w/o ".txt"
            if (isSafeString(namebody)) namebody = decodeSafeString(namebody);
            var li = document.createElement("li");
            var a = document.createElement("a");
            var t = document.createTextNode(namebody);
            a.setAttribute("href", "#");
            a.setAttribute("id", optionHeader+id);
            $(a).click(function (evt) {
                loadOpen(evt);
            });
            li.appendChild(a);
            a.appendChild(t);
            $("#LocalFileList").prepend(li);
        }
        $('#LocalFileList').listview('refresh');
        loadingIndicatorOff();
    }, function myfunction(e) {
        loadingIndicatorOff();
        alert(e.error.message);
    });
}

function processJson(json) {
    var ar = json.kml.Document.Placemark;
    for (var i = 0; i < ar.length; i++) {
        addMarker2(ar[i]);
    }
    if (ar.length > 0) {
        var v = ar[ar.length-1].Point.coordinates.split(",");
        centerrize(v[0], v[1]);
    }
}

function loadFromSkyDrive(elementid) {
    loadingIndicatorOn();
    var id = elementid.substring(optionHeader.length, elementid.length);
    loginForWrite(function (r) {
        WL.api({
            path: id + "/content",
            method: "GET"
        }).then(
            function (response) {
                $.cookie('mapLoadUrl', response.location);
                $.cookie('fileid', id);
                location.reload(true);
            },
            function (responseFailed) {
                loadingIndicatorOff();
                alert("Error calling API: " + responseFailed.error.message);
            }
        );
    });
}

function onJsonpLoaded(json) {
    currentPlots = json;
}

function setPlots() {
    if (currentPlots) processJson(currentPlots);
    else
    {
        var args = parseArg();
        var s = args["sourceurl"];
        if (!s) s = args["sourceUrl"];
        if (s) {
            $.cookie('mapLoadUrl', encodeURI(s));
            location.reload(true);
        }
        else {
            currentPlots = { 'kml': { 'Document': { 'Placemark': [] } } };
        }
    }
}

function xhrResultReport(xhr) {
    if (xhr.status == 200)
        alert("Saved");
    else
        alert("Not Saved: " + xhr.statusText);
}

function saveOverwrite() {
    if (!currentPlots) {
        alert("No file to save.");
        return;
    }
    if (!currentFileId) {
        alert("No file to overwrite. Place try 'Save to NEW File'.");
        return;
    }
    if (!window.confirm('Overwrite?')) return;
    loadingIndicatorOn();
    getFolderId(function (folderid) {
        var xhr = new XMLHttpRequest();
        xhr.open("POST", "UploadProxy.aspx?id=" + currentFileId + "&folderid=" + folderid + "&access_token=" + getAccessToken(), false);
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4) {
                loadingIndicatorOff();
                xhrResultReport(xhr);
            }
        }
        xhr.send("onJsonpLoaded(\r\n" + JSON.stringify(currentPlots)+");");
    });
}

function saveAs() {
    var name = $("#SaveAsName").val();
    if (!name || name == "") {
        alert("Enter Name First");
        return;
    }
    loadingIndicatorOn();
    getFolderId(function (folderid) {
        var xhr = new XMLHttpRequest();
        xhr.open("POST", "UploadProxy.aspx?newname=" + encodeURI(encodeSafeString(name)) + "&folderid=" + folderid + "&access_token=" + getAccessToken(), false);
        xhr.onreadystatechange = function () {
            if (xhr.readyState == 4) {
                loadingIndicatorOff();
                saveClose();
                xhrResultReport(xhr);
            }
        }
        xhr.send("onJsonpLoaded(\r\n" + JSON.stringify(currentPlots) + ");");
    });
}

function SaveNewCmd() {
    if (!currentPlots) {
        alert("No file to save.");
        return;
    }
    $("#savemask").css("visibility", "visible");
    $("#SaveAsName").val(nameFixer($("#SaveAsName").val()));
}

function explainCmd() {
    $("#explainmask").css("visibility", "visible");
}
function listCmd() {
    var ar = currentPlots.kml.Document.Placemark;
    $("#listul").empty();
    for (var i = 0; i < ar.length; i++) {
        (function () {
            var v = ar[i].Point.coordinates.split(",");
            var a = document.createElement("a");
            $(a).attr("href", "#");
            $(a).click(function () {
                centerrize(v[0], v[1]);
                $("#listmask").css("visibility", "hidden");
            });
            $(a).text(ar[i].name + " (" + ar[i].Point.coordinates + ")");
            var li = document.createElement("li");
            $(li).append(a);
            $("#listul").append(li);
        })();
    }
    $('#listul').listview('refresh');
    $("#listmask").css("visibility", "visible");
}
function viewAllCmd() {
    var latMin=90, latMax=0;
    var lngMin=180, lngMax=-180;
    var ar = currentPlots.kml.Document.Placemark;
    if (ar.length < 2) {
        alert("You required at least 2 points.");
        return;
    }
    for (var i = 0; i < ar.length; i++) {
        var v = ar[i].Point.coordinates.split(",");
        var lat = v[1];
        var lng = v[0];
        if (lat < latMin) latMin = lat;
        if (lat > latMax) latMax = lat;
        if (lng < lngMin) lngMin = lng;
        if (lng > lngMax) lngMax = lng;
    }
    var sw = new Y.LatLng(latMin, lngMin);
    var ne = new Y.LatLng(latMax, lngMax);
    var bounds = new Y.LatLngBounds(sw, ne);
    yahoomap.drawBounds(bounds);
}

function setDefaultPlaceHereCmd() {
    $.cookie('plt2lat', yahoomap.getCenter().lat(), { expires: new Date(2030, 1) });
    $.cookie('plt2lon', yahoomap.getCenter().lng(), { expires: new Date(2030, 1) });
}

function adjustSize() {
    if ($("#openCommands").innerHeight() == 0) {
        setTimeout(function () {
            adjustSize();
            if (yahoomap && yahoomap.updateSize)
                yahoomap.updateSize()

        }, 100);
        return;
    }
    var toolh = $("#openCommands").innerHeight();
    var w = window.innerWidth;
    var h = window.innerHeight;
    $("#outer").css("top", toolh + "px");
    $("#plottarget").css("top", toolh + "px");
    $("#loading").css("left", (w / 2) + "px");
    $("#loading").css("top", (h / 2) + "px");
    $(".womask").css("width", w + "px");
    $(".womask").css("height", (h - toolh) + "px");
    $(".mask").css("width", w + "px");
    $(".mask").css("height", h + "px");
}

function redirectForDebug() {
    var url = location.href.toLowerCase();
    if (url.indexOf("127.0.0.1") >= 0) {
        location.href = url.replace("127.0.0.1", "pd106.piedey.co.jp");
        return true;
    }
    if (url.indexOf("localhost") >= 0) {
        location.href = url.replace("localhost", "pd106.piedey.co.jp");
        return true;
    }
    return false;
}

function loadOpen(evt) {
    var src = evt.srcElement;
    if (!src) src = evt.target;
    loadFromSkyDrive(src.id);
    loadClose();
}

function addPoint(ll) {
    var rawnewname = createDefaultAlterName();
    var div1 = document.createElement("div");
    var div2 = document.createElement("div");
    var t = document.createTextNode(rawnewname);
    div2.appendChild(t);
    div1.appendChild(div2);
    var newname = $(div1).html();
    var pntObj = { "name": rawnewname, "description": newname, "Point": { "coordinates": ll.Lon + "," + ll.Lat } };
    currentPlots.kml.Document.Placemark.push(pntObj);
    addMarker2(pntObj);
    return pntObj;
}

$(document).bind("mobileinit", function () {
    $.mobile.pushStateEnabled = false;
    $.mobile.ajaxEnabled = false;
    $.mobile.hashListeningEnabled = false;
    $.mobile.autoInitializePage = false;
});

$(document).ready(function () {
    $.mobile.initializePage();
});

$('#mymainbody').live('pageinit', function (event) {
    if (redirectForDebug()) return;
    currentFileId = $.cookie('fileid');
    adjustSize();
    yahoomap = new Y.Map("map_canvas");
    var x = 35.69;
    var y = 139.699583;
    if( $.cookie('plt2lat') ) x = $.cookie('plt2lat');
    if( $.cookie('plt2lon') ) y = $.cookie('plt2lon');
    yahoomap.drawMap(new Y.LatLng(x, y), 17, Y.LayerSetId.NORMAL);
    yahoomap.addControl(new Y.LayerSetControl());
    yahoomap.addControl(new Y.SliderZoomControlVertical());
    $("#map_canvas").mousedown(function (evt) {
        if (isMoveOn() && currentPropertyObject) {
            var old = currentPropertyObject.Point.coordinates;
            var pos = $("#outer").position();
            var x = evt.pageX - pos.left;
            var y = evt.pageY - pos.top;
            var ll = yahoomap.fromContainerPixelToLatLng(new Y.Point(x, y));
            currentPropertyObject.Point.coordinates = ll.Lon+","+ll.Lat;
            yahoomap.clearFeatures();
            var ar = currentPlots.kml.Document.Placemark;
            for (var i = 0; i < ar.length; i++) {
                addMarker2(ar[i]);
            }
            moveOff();
        }
    });
    $("#PenCmd").click(function () {
        $("#plottarget").css("visibility", "visible");
    });
    $("#plottarget").mousedown(function (evt) {
        var pos = $(this).position();
        var x = evt.pageX - pos.left;
        var y = evt.pageY - pos.top;
        var ll = yahoomap.fromContainerPixelToLatLng(new Y.Point(x, y));
        addPoint(ll);
        $("#plottarget").css("visibility", "hidden");
    });
    $("#NewCmd").click(function () {
        if (window.confirm("I'll clear all unsaved plots. OK?")) {
            $.cookie('mapLoadUrl', null);
            location.reload();
        }
    });
    $("#SaveCmd").click(function () {
        saveOverwrite();
    });
    $("#SaveNewCmd").click(function () {
        SaveNewCmd();
    });
    $("#OpenCmd").click(function () {
        $("#loadmask").css("visibility", "visible");
        loadingIndicatorOn();
        getFolderId(function (folderid) {
            if (!folderid) return;
            initSkyDriveFileList(folderid);
        });
    });
    $("#SaveClose").click(function () {
        saveClose();
    });
    $("#JpegCmd").click(function () {
        jpegOn();
    });
    $("#propertyEdit").click(function () {
        propertyOff();
        editOn(currentPropertyObject);
    });
    $("#editOK").click(function () {
        editSave();
        editOff();
    });
    $("#editCancel").click(function () {
        editOff();
    });
    $("#editUpload").click(function () {
        editUpload();
    });
    $("#editDelete").click(function () {
        editDelete();
    });
    $("#SaveClose").click(function () {
        saveClose();
    });
    $("#LoadClose").click(function () {
        loadClose();
    });
    $("#SaveAs").click(function () {
        saveAs();
    });
    $("#ExplainCmd").click(function () {
        explainCmd();
    });
    $("#ListCmd").click(function () {
        listCmd();
    });
    $("#ViewAllCmd").click(function () {
        viewAllCmd();
    });
    $("#SetDefaultPlaceHereCmd").click(function () {
        setDefaultPlaceHereCmd();
    });
    $("#openCommands").click(function () {
        if ($("#toolPalette").css("visibility") == "visible" || $("#toolPalette2").css("visibility") == "visible")
            commandsClose();
        else
            $("#toolPalette").css("visibility", "visible");
    });
    $("#GoToTools2").click(function () {
        $("#toolPalette").css("visibility", "hidden");
        $("#toolPalette2").css("visibility", "visible");
    });
    $("#closeCommands").click(function () {
        commandsClose();
    });
    $("#closeCommands2").click(function () {
        commandsClose();
    });
    $("#propertyClose").click(function () {
        propertyOff();
    });
    $("#editMove").click(function () {
        editOff();
        moveOn();
    });
    $("#editOK").click(function () {
        editOff();
    });
    $("#editCancel").click(function () {
        editOff();
    });
    $("#jpegUpload").click(function () {
        jpegUpload();
    });
    $("#jpegOK").click(function () {
        jpegOff();
    });
    $("#jpegCancel").click(function () {
        jpegOff();
    });
    $("#explainClose").click(function () {
        $("#explainmask").hide();
    });
    $("#listCancel").click(function () {
        $("#listmask").hide();
    });
    $("#cancelMove").click(function () {
        moveOff();
    });
    $(document).keypress(function (e) {
        if (e.which == 27 && isMoveOn()) {
            moveOff();
        }
    });
    $(".stdtool").click(function () {
        commandsClose();
    });
    setPlots();
});
$(window).resize(function () {
    adjustSize();
    if (yahoomap && yahoomap.updateSize) 
        yahoomap.updateSize()
});
